% Estimation using the BEA data compiled by Pesaran and Yang (2017)
% Results are presented in Tables 9, 10 and 11 in the paper
clear all;
close all;
clc;
fpath = '.\..';       % file path
% fname = strcat(fpath, '\data\W_1972-2007.xlsx'); 
% W_1972 = xlsread(fname,'W_1972');
% W_1977 = xlsread(fname,'W_1977');
% W_1982 = xlsread(fname,'W_1982');
% W_1987 = xlsread(fname,'W_1987');
% W_1992 = xlsread(fname,'W_1992');
% W_1997 = xlsread(fname,'W_1997');
% W_2002 = xlsread(fname,'W_2002');
% W_2007 = xlsread(fname,'W_2007');
% save(strcat(fpath, '\data\W_1972-2007.mat'));

% Alternatively, load .mat file directly
load(strcat(fpath, '\data\W_1972-2007.mat'));

%% Cross section estimation
T = 8;
M = 5;    % report the top five pervasive sectors
deltaM_CStab = zeros(T,M);
Nnz_CStab = zeros(T,1);
idx_deltaM_CStab = zeros(T,M);
for t = 1:T
    yr = 1972 + (t-1)*5;    
    W = eval(strcat('W_', num2str(yr)));
    N = size(W,1);
    dN = sum(W,1);     
    CSresults = est_delta(dN,M); 
    deltaM_CStab(t,:) = CSresults.deltaM; 
    Nnz_CStab(t) = CSresults.Nnz;    % number of units with non-zero outdegrees
    idx_deltaM_CStab(t,:) = CSresults.idx_deltaM;  % index array
end
table_CS = [Nnz_CStab,deltaM_CStab];  % Table 9 in the paper
% fname = strcat(fpath,'\output\resultsCS_BEAdata.csv');
% csvwrite(fname,table_CS,0,0);
% display output
disp(' ');
disp('==============================');
disp('Table 9: Yearly estimates');
fmtspec = repmat(strcat('%d',repmat(' %1.3f ',1,M),'\n'),1,T);
fprintf(fmtspec, table_CS');   

%%
% find out the identities of top pervasive sectors using the index array
fname = strcat(fpath,'\data\W_1972-2007_id.xlsx');
[~,txt] = xlsread(fname, 'W_1972');
sectorM_name_1972 = txt(idx_deltaM_CStab(1,:));
[~,txt] = xlsread(fname, 'W_1977');
sectorM_name_1977 = txt(idx_deltaM_CStab(2,:));
[~,txt] = xlsread(fname, 'W_1982');
sectorM_name_1982 = txt(idx_deltaM_CStab(3,:));
[~,txt] = xlsread(fname, 'W_1987');
sectorM_name_1987 = txt(idx_deltaM_CStab(4,:));
[~,txt] = xlsread(fname, 'W_1992');
sectorM_name_1992 = txt(idx_deltaM_CStab(5,:));
[~,txt] = xlsread(fname, 'W_1997');  % BEA codes contain strings since year 1997
txt2 = txt(:,2);
sectorM_name_1997 = txt2(idx_deltaM_CStab(6,:));
[~,txt] = xlsread(fname, 'W_2002');
txt2 = txt(:,2);
sectorM_name_2002 = txt2(idx_deltaM_CStab(7,:));
[~,txt] = xlsread(fname, 'W_2007');
txt2 = txt(:,2);
sectorM_name_2007 = txt2(idx_deltaM_CStab(8,:));
% display output
disp(' ');
disp('==============================');
disp('Table 10: Identities based on the yearly estimates');
for t = 1:T
    yr = 1972 + (t-1)*5;   
    disp(num2str(yr));
    disp(eval(strcat('sectorM_name_', num2str(yr))));
end

%% Unbalanced panel estimation, 1972-1992
fname = strcat(fpath,'\data\dTN_panel_1972-1992.csv');
dTN1 = csvread(fname); 
P1results = est_delta(dTN1,M); 
fid = fopen(strcat(fpath,'\data\dTN_panel_1972-1992_id.csv'),'rt');
P1_id = textscan(fid,'%s %q','delimiter',',');
fclose(fid);
P1_name = P1_id{:,2};
disp(' ');
disp('==============================');
disp('Table 11: Sub-sample 1972-1992');
fprintf('Degrees of dominance: %1.3f %1.3f %1.3f %1.3f %1.3f\n', P1results.deltaM);
fprintf('Standard error: %1.3f\n', P1results.se_deltaM);
disp('Identities');
disp(P1_name(P1results.idx_deltaM));   
fprintf('Number of sectors with non-zero outdegrees: %d\n',P1results.Nnz)

%% Unbalanced panel estimation, 1997-2007
fname = strcat(fpath,'\data\dTN_panel_1997-2007.csv');
dTN2 = csvread(fname); 
P2results = est_delta(dTN2,M); 
fid = fopen(strcat(fpath,'\data\dTN_panel_1997-2007_id.csv'),'rt');
P2_id = textscan(fid,'%s %q','delimiter',',');
fclose(fid);
P2_name = P2_id{:,2};
P2_name(P2results.idx_deltaM);
disp(' ');
disp('==============================');
disp('Table 11: Sub-sample 1997-2007');
fprintf('Degrees of dominance: %1.3f %1.3f %1.3f %1.3f %1.3f\n', P2results.deltaM);
fprintf('Standard error: %1.3f\n', P2results.se_deltaM);
disp('Identities');
disp(P2_name(P2results.idx_deltaM));   
fprintf('Number of sectors with non-zero outdegrees: %d\n',P2results.Nnz)


